"""Proxy harvester module"""
import concurrent.futures
import logging
import re
import requests


def fetch_list(url):
    try:
        response = requests.get(url)
        response.raise_for_status()
    except requests.RequestException:
        logging.warning(f'Error fetching proxies from {url}')
        return []

    proxy_regex = r"(?:\b(?:[\S]+:)?(?:[\S]+)?@\b)?(?:\d{1,3}\.){3}\d{1,3}:\d+"
    proxies = re.findall(proxy_regex, response.text)
    logging.info(f'Fetched {len(proxies)} proxies from {url}')
    return proxies


def fetch_all(urls, max_workers=8):
    proxies = []
    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
        future_to_url = {executor.submit(fetch_list, url): url for url in urls}
        for future in concurrent.futures.as_completed(future_to_url):
            url = future_to_url[future]
            try:
                proxy_list = future.result()
                for proxy in proxy_list:
                    if proxy not in proxies:
                        proxies.append(proxy)
            except:
                pass

    logging.info(f'Fetched {len(proxies)} proxies in total')
    return proxies


def validate(proxy, type='http'):
    # Check regex
    return None
